package com.flyfish.framework.beans.repository;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.beans.factory.FactoryBean;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.support.AbstractBeanDefinition;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.BeanDefinitionRegistry;
import org.springframework.beans.factory.support.BeanNameGenerator;
import org.springframework.core.env.Environment;
import org.springframework.core.io.ResourceLoader;
import org.springframework.core.type.AnnotationMetadata;
import org.springframework.data.mongodb.repository.config.EnableMongoRepositories;
import org.springframework.data.mongodb.repository.config.MongoRepositoryConfigurationExtension;
import org.springframework.data.repository.config.*;

/**
 * 自定义的仓库注册器
 * @author wangyu
 */
public class CustomRepositoryRegistrar {

    // repo生成的extension
    private final RepositoryConfigurationExtension extension = new MongoRepositoryConfigurationExtension();
    private final RepositoryConfigurationSource configurationSource;
    private final CustomRepositoryBuilder builder;

    public CustomRepositoryRegistrar(AnnotationMetadata metadata, ResourceLoader resourceLoader, Environment environment,
                                     BeanDefinitionRegistry registry, BeanNameGenerator generator) {
        // repo生成的构建器
        this.configurationSource = new AnnotationRepositoryConfigurationSource(metadata,
                EnableMongoRepositories.class, resourceLoader, environment, registry, generator);
        // 构建器
        this.builder = new CustomRepositoryBuilder(registry, extension,
                configurationSource, resourceLoader, environment);

    }

    /**
     * 根据class直接注册bean
     * @param clazz 编译后的class
     */
    public void register(Class<?> clazz) {
        //生成BeanDefinition并注册到容器中
        BeanDefinition origin = BeanDefinitionBuilder.genericBeanDefinition(clazz).getRawBeanDefinition();
        //设置当前bean定义对象是单例的
        origin.setScope("singleton");
        // 构造配置
        RepositoryConfiguration<? extends RepositoryConfigurationSource> configuration =
                new DefaultRepositoryConfiguration<>(configurationSource, origin, extension);
        BeanDefinitionBuilder definitionBuilder = builder.build(configuration);
        extension.postProcess(definitionBuilder, configurationSource);
        extension.postProcess(definitionBuilder, (AnnotationRepositoryConfigurationSource) configurationSource);


        AbstractBeanDefinition beanDefinition = definitionBuilder.getBeanDefinition();
        beanDefinition.setPrimary(configuration.isPrimary());
        String beanName = StringUtils.uncapitalize(clazz.getSimpleName());
        beanDefinition.setAttribute(FactoryBean.OBJECT_TYPE_ATTRIBUTE, configuration.getRepositoryInterface());
        builder.getRegistry().registerBeanDefinition(beanName, beanDefinition);
    }

    public RepositoryConfigurationExtension getExtension() {
        return extension;
    }
}
